import sys, os, string
import unittest

from Products.ZOQLMethod.Parser import ZOQLParser

### This class tests the different Forms of CREATE
### add as much test methods as possible

class CreateTest(unittest.TestCase):

    def setUp(self):
        
        self.parser = ZOQLParser.ZOQLParser(marshalfile="oql.mar", bind=0)
        self.context = {}


    ### Tests for product creation
        
    def testCreatePythonProduct(self):
        """Python product creation"""

        try:
            self.parser.parse("CREATE PRODUCT Test AS PYTHON;", self.context)
        except:
            self.fail("Failed to parse python product creation")

    def testCreateZClassProduct(self):
        """ZClass product creation"""
        
        try:
            self.parser.parse("CREATE PRODUCT Test AS ZCLASS;", self.context)
        except:
            self.fail("Failed to parse zclass product creation")

    def testCreateProductOthers(self):
        """Wrong product type"""
        
        # wrong product type
        
        self.failUnlessRaises(SyntaxError, self.parser.parse,
                              "CREATE PRODUCT Test AS OTHER;",
                              self.context
                              )

    def testCreateProductKeywordName(self):
        """Keyword as product name"""
        
        self.failUnlessRaises(SyntaxError, self.parser.parse,
                              "CREATE PRODUCT CLASS AS PYTHON;",
                              self.context
                              )
        
    def testCreateProductZopishName(self):
        """Zopish Product names not allowed"""
        
        self.failUnlessRaises(SyntaxError, self.parser.parse,
                              "CREATE PRODUCT 'A Test' AS OTHER;",
                              self.context
                              )

    ### Tests for class creation
        
    def testCreateClassSimple(self):
        """Test CREATE CLASS statement"""

        try:
            self.parser.parse("CREATE CLASS Test IN Testproduct;", self.context)
        except:
            self.fail("CREATE CLASS Test IN Testproduct;")

    def testCreateClassInZopish(self):
        """Class creation in zopish product"""
        
        # wrong product type
        
        self.failUnlessRaises(SyntaxError, self.parser.parse,
                              "CREATE CLASS Test IN z'A Testproduct';",
                              self.context
                              )
        
    # Inheritance

    def testCreateClassInheritedSimple(self):
        """Class creation with simple inheritance"""

        try:
            self.parser.parse("CREATE CLASS Test IN Testproduct INHERIT AClass;", self.context)
        except:
            self.fail("CREATE CLASS Test IN Testproduct INHERIT AClass;")

    def testCreateClassInheritedPathMultiple(self):
        """Class creation with multiple inheritance from path"""
        try:
            self.parser.parse("CREATE CLASS Test IN Testproduct INHERIT A.Klass.Path, B.Klass.Path;", self.context)
        except:
            self.fail("CREATE CLASS Test IN Testproduct INHERIT A.Klass.Path, B.Klass.Path;")


    def testCreateClassInheritedPathMultipleZopishMixed(self):
        """Class creation with inheritance from zopish path"""
        try:
            self.parser.parse("CREATE CLASS Test IN Testproduct INHERIT A.Klass.Path, z'B Klass'.Path;", self.context)
        except:
            self.fail("CREATE CLASS Test IN Testproduct INHERIT A.Klass.Path, z'B Klass'.Path;")


    # Options
    
    def testCreateClassOption(self):
        """Class creation with option"""

        try:
            self.parser.parse("CREATE CLASS Test IN Testproduct OPTION a=b;", self.context)
        except:
            self.fail("CREATE CLASS Test IN Testproduct OPTION a=b;")

    def testCreateClassOptionList(self):
        """Class creation with option list"""
        
        try:
            self.parser.parse("CREATE CLASS Test IN Testproduct OPTION a=b, c+=d;", self.context)
        except:
            self.fail("CREATE CLASS Test IN Testproduct OPTION a=b, c+=d;")

    def testCreateClassOptionListZopish(self):
        """Class creation with zopish option list"""
        
        try:
            self.parser.parse("CREATE CLASS Test IN Testproduct OPTION z'a var'=b, c+=4;", self.context)
        except:
            self.fail("CREATE CLASS Test IN Testproduct OPTION z'a var'=b, c+=d;")



    # Property Sheet Declarations
    
    def testCreateClassPropertyEmpty(self):
        """Class creation with empty property sheet"""

        try:
            self.parser.parse("CREATE CLASS Test IN Testproduct DECLARE A {};", self.context)
        except:
            self.fail("CREATE CLASS Test IN Testproduct DECLARE A {};")


    def testCreateClassPropertiesList(self):
        """Class creation with property sheet list"""

        try:
            self.parser.parse("CREATE CLASS Test IN Testproduct DECLARE A{}, B{};", self.context)
        except:
            self.fail("CREATE CLASS Test IN Testproduct DECLARE A{}, B{};")

    def testCreateClassPropertiesListZopish(self):
        """Class creation with zopish property sheet list"""
        
        try:
            self.parser.parse("CREATE CLASS Test IN Testproduct DECLARE 'Zope Sheet' {}, B {};", self.context)
        except:
            self.fail("CREATE CLASS Test IN Testproduct DECLARE 'Zope Sheet' {}, B{};")


    def testCreateClassProperty(self):
        """Class creation with non empty property sheet"""

        try:
            self.parser.parse("CREATE CLASS Test IN TestProduct DECLARE Sheet {id:string, title:string};")
        except:
            self.fail("CREATE CLASS Test IN TestProduct DECLARE Sheet {id:string, title:string};")

    def testCreateClassPropertyDefault(self):
        """Class creation with non empty property sheet and defaults"""

        try:
            self.parser.parse("CREATE CLASS Test IN TestProduct DECLARE Sheet {id:string DEFAULT 'ident', title:string DEFAULT ''};")
        except:
            self.fail("CREATE CLASS Test IN TestProduct DECLARE Sheet {id:string DEFAULT 'ident', title:string DEFAULT ''};")

    def testCreateClassPropertyDefaultFunction(self):
        """Class creation with defaults from function"""

        try:
            self.parser.parse("CREATE CLASS Test IN TestProduct DECLARE Sheet {id:string DEFAULT get_id(), title:string DEFAULT ''};")
        except:
            self.fail("CREATE CLASS Test IN TestProduct DECLARE Sheet {id:string DEFAULT get_id(), title:string DEFAULT ''};")

    # Sequences

    def testCreateSimpleSequence(self):
        """Simple Sequence creation"""

        try:
            self.parser.parse("CREATE SEQUENCE seq;")
        except:
            self.fail("CREATE SEQUENCE seq;")


    def testCreateSequenceWithIncrement(self):
        """Sequence creation with increment"""

        try:
            self.parser.parse("CREATE SEQUENCE seq INCREMENT 2;")
        except:
            self.fail("CREATE SEQUENCE seq INCREMENT 2;")


    def testCreateSequenceWithMinvalue(self):
        """Sequence creation with minvalue"""

        try:
            self.parser.parse("CREATE SEQUENCE seq MINVALUE 0;")
        except:
            self.fail("CREATE SEQUENCE seq MINVALUE 0;")


    def testCreateSequenceWithMaxvalue(self):
        """Sequence creation with maxvalue"""

        try:
            self.parser.parse("CREATE SEQUENCE seq MAXVALUE 100;")
        except:
            self.fail("CREATE SEQUENCE seq MAXVALUE 100;")


    def testCreateSequenceWithStart(self):
        """Sequence creation with start"""

        try:
            self.parser.parse("CREATE SEQUENCE seq START 3;")
        except:
            self.fail("CREATE SEQUENCE seq START 3;")


    def testCreateSequenceWithAllOptions(self):
        """Sequence creation with all options"""

        try:
            self.parser.parse("CREATE SEQUENCE seq INCREMENT 2 MINVALUE 0 MAXVALUE 100 START 4;")
        except:
            self.fail("CREATE SEQUENCE seq INCREMENT 2 MINVALUE 0 MAXVALUE 100 START 4;")

